from random import randrange
from logging import getLogger
from . import models

# generate_random_code
CHARSET = '0123456789ABCDEFGHJKMNPQRSTVWXYZ'
DEFAULT_LENGTH = 16
DEFAULT_MAX_TRIES = 32

# get_default_logging_group
DEFAULT_LOGGING_GROUP_CODE='DEFAULT'

logger = getLogger(__name__)


def get_current_app_from_request(request):
    try:
        api_key = request.META["HTTP_API_KEY"]
        app = models.App.objects.get(api_key=api_key, enabled=True)
        return app or None

    except Exception as e:
        logger.error(e)
        return None


def generate_random_code(length=DEFAULT_LENGTH, max_tries=DEFAULT_MAX_TRIES):
    code = ""
    loop_num = 0
    unique = False

    while not unique:
        if loop_num < max_tries:
            new_code = ''
            for i in range(length):
                new_code += CHARSET[randrange(0, len(CHARSET))]
                code = new_code
                unique = True
            loop_num += 1
            return code
        else:
            raise ValueError("Couldn't generate a unique code.")


def get_default_logging_group(code=DEFAULT_LOGGING_GROUP_CODE):
    return models.LoggingGroup.objects.get(code=code).pk



# Generated by Django 2.1.2 on 2018-11-20 04:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rest_framework_apicontrol', '0008_loggingevent_logging_group'),
    ]

    operations = [
        migrations.AddField(
            model_name='loggingevent',
            name='origin',
            field=models.CharField(choices=[('auto', 'auto'), ('user', 'user'), ('rest', 'rest')], default='auto', max_length=128),
        ),
    ]



# Generated by Django 2.1.2 on 2018-11-20 02:54

from django.db import migrations, models
import rest_framework_apicontrol.utils


class Migration(migrations.Migration):

    dependencies = [
        ('rest_framework_apicontrol', '0005_loggingevent_logginggroup'),
    ]

    operations = [
        migrations.AddField(
            model_name='logginggroup',
            name='code',
            field=models.CharField(default=rest_framework_apicontrol.utils.generate_random_code, max_length=128),
        ),
    ]



"""rest_framework_apicontrol context processors."""


def available_apps(request):
    """Return the available apps for the current user."""
    available_apps = None

    if request.user.is_authenticated:
        available_apps = request.user.app_set.all()

    return {
        'AVAILABLE_APPS': available_apps,
    }



# Generated by Django 2.0.3 on 2018-03-28 01:52

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('rest_framework_apicontrol', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='organizationalunit',
            old_name='unit_type',
            new_name='organizational_unit_type',
        ),
        migrations.AddField(
            model_name='app',
            name='description',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='app',
            name='metadata',
            field=django.contrib.postgres.fields.jsonb.JSONField(default={}),
        ),
        migrations.AddField(
            model_name='app',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='app',
            name='unique_id',
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AddField(
            model_name='organizationalunit',
            name='unique_id',
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AddField(
            model_name='organizationalunittype',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='organizationalunittype',
            name='unique_id',
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AddField(
            model_name='organizationalunittype',
            name='updated_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='responsible',
            name='app',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='rest_framework_apicontrol.App'),
        ),
        migrations.AddField(
            model_name='responsible',
            name='unique_id',
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='app',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='app',
            name='name',
            field=models.CharField(blank=True, max_length=128),
        ),
        migrations.AlterField(
            model_name='app',
            name='updated_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='organizationalunit',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='organizationalunit',
            name='description',
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name='organizationalunit',
            name='name',
            field=models.CharField(max_length=128),
        ),
        migrations.AlterField(
            model_name='organizationalunit',
            name='updated_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='organizationalunittype',
            name='app',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='rest_framework_apicontrol.App'),
        ),
        migrations.AlterField(
            model_name='organizationalunittype',
            name='description',
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name='organizationalunittype',
            name='name',
            field=models.CharField(max_length=128),
        ),
        migrations.AlterField(
            model_name='responsible',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='responsible',
            name='email',
            field=models.EmailField(max_length=254),
        ),
        migrations.AlterField(
            model_name='responsible',
            name='name',
            field=models.CharField(blank=True, default='RESPONSIBLE', max_length=128),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='responsible',
            name='updated_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterUniqueTogether(
            name='responsible',
            unique_together={('email', 'app')},
        ),
    ]



"""rest_framework_apicontrol model mixins."""
import uuid
from django.db import models
from django.utils import timezone

# Fields constants
STATUS_FIELD_MAX_LENGTH = 32

# Model instances status field choices
MODEL_INSTANCE_STATUS_CHOICES = (
    ('active', 'active'),
    ('inactive', 'inactive'),
    ('archived', 'archived'),
    ('removed_by_user', 'removed_by_user'),
    ('removed_by_admin', 'removed_by_admin'),
    ('removed_automatically', 'removed_automatically'),
)


class ActiveModelMixin(models.Model):
    """
    ActiveModelMixin.

    It's a model mixin to set if a model instance is active or not.
    """

    active = models.BooleanField(
        default=False
    )

    def activate(self, commit=True):
        """Activate the instance."""
        self.active = True

        if commit:
            self.save()

    def deactivate(self, commit=True):
        """Deactivate the instance."""
        self.active = False

        if commit:
            self.save()

    class Meta:
        abstract = True


class EnabledModelMixin(models.Model):
    """
    EnabledModelMixin.

    It's a model mixin to set if a model instance is enabled or not.
    """

    enabled = models.BooleanField(
        default=False
    )

    def enable(self, commit=True):
        """Enable the instance."""
        self.enabled = True

        if commit:
            self.save()

    def disable(self, commit=True):
        """Disable the instance."""
        self.enabled = False

        if commit:
            self.save()

    class Meta:
        abstract = True


class PerAppModelMixin(models.Model):
    """
    PerAppModelMixin.

    It's a model mixin that allows a model to be associated with an app.
    """

    app = models.ForeignKey(
        'rest_framework_apicontrol.app',
        on_delete=models.CASCADE,
        null=True
    )

    class Meta:
        abstract = True


class InstanceStatusModelMixin(models.Model):
    """
    InstanceStatusModelMixin.

    It's a model mixin to set an status to a Model Instance.
    """

    instance_status = models.CharField(
        max_length=STATUS_FIELD_MAX_LENGTH,
        choices=MODEL_INSTANCE_STATUS_CHOICES,
        default='active'
    )

    def activate_instance(self, commit=True):
        """Activate the instance."""
        self.instance_status = 'active'

        if commit:
            self.save()

    def deactivate_instance(self, commit=True):
        """Deactivate the instance."""
        self.instance_status = 'inactive'

        if commit:
            self.save()

    def archive_instance(self, commit=True):
        """Archive the instance."""
        self.instance_status = 'archived'

        if commit:
            self.save()

    # The following statuses are not implemented as functions(maybe in the
    # future if needed):
    # 'removed_by_user'
    # 'removed_by_admin'
    # 'removed_automatically'

    class Meta:
        abstract = True


class TrackableModelMixin(models.Model):
    """
    TrackableModelMixin.

    It's a model mixin to track when a model instance is created
    or updated.
    """

    created_at = models.DateTimeField(
        default=timezone.now
    )
    updated_at = models.DateTimeField(
        default=timezone.now
    )

    class Meta:
        abstract = True


class UniqueIDModelMixin(models.Model):
    """
    UniqueIDModelMixin.

    It's a model mixin to set the model unique_id field as an UUIDField with
    UUID4 algorithm.
    """

    unique_id = models.UUIDField(
        default=uuid.uuid4,
        editable=False
    )

    class Meta:
        abstract = True



# Generated by Django 2.1.2 on 2018-11-20 02:56

import django.db.models.deletion
from django.db import migrations, models
from rest_framework_apicontrol.utils import (
    DEFAULT_LOGGING_GROUP_CODE,
    get_default_logging_group
)

def generate_default_logging_group(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    LoggingGroup = apps.get_model("rest_framework_apicontrol", "LoggingGroup")
    default_logging_group, created = LoggingGroup.objects.get_or_create(
        code=DEFAULT_LOGGING_GROUP_CODE
    )

def remove_default_logging_group(apps, schema_editor):
    LoggingGroup = apps.get_model("rest_framework_apicontrol", "LoggingGroup")
    LoggingGroup.objects.filter(code=DEFAULT_LOGGING_GROUP_CODE).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('rest_framework_apicontrol', '0006_logginggroup_code'),
    ]

    operations = [
        migrations.RunPython(
            generate_default_logging_group,
            remove_default_logging_group
        ),
    ]



# Generated by Django 2.1.2 on 2018-11-20 02:26

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.utils.timezone
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('rest_framework_apicontrol', '0004_auto_20181029_1933'),
    ]

    operations = [
        migrations.CreateModel(
            name='LoggingEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('unique_id', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('name', models.CharField(blank=True, max_length=128)),
                ('description', models.TextField(blank=True)),
                ('metadata', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LoggingGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('unique_id', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('name', models.CharField(blank=True, max_length=128)),
                ('description', models.TextField(blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 2.0.3 on 2018-04-03 05:12

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('rest_framework_apicontrol', '0002_auto_20180328_0152'),
    ]

    operations = [
        migrations.AlterField(
            model_name='app',
            name='metadata',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}),
        ),
    ]



# from distutils.core import setup
from setuptools import setup
from os import path


# Read the README.md file content
def get_long_description(filename='README.md'):
    this_directory = path.abspath(path.dirname(__file__))
    with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
        return f.read()


setup(
    name='rest_framework_apicontrol',
    packages=['rest_framework_apicontrol'],
    include_package_data=True,
    version='0.7.7',
    description='Django Rest Framework Library to control Client Apps over APIs',
    long_description=get_long_description(),
    long_description_content_type='text/markdown',
    author='Marcelo Cueto',
    author_email='cueto@live.cl',
    url='https://github.com/mcueto/djangorestframework-apicontrol',
    download_url='https://github.com/mcueto/djangorestframework-auth0/tarball/0.7.7',
    keywords=['api', 'control', 'rest framework', 'django'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
    ],
    install_requires=[
        'django>=2.0.0',
        'djangorestframework>=3.7.0'
        'psycopg2>=2.7.3'
    ],
)



# Generated by Django 2.1.2 on 2018-11-20 03:18

from django.db import migrations, models
import django.db.models.deletion
import rest_framework_apicontrol.utils


class Migration(migrations.Migration):

    dependencies = [
        ('rest_framework_apicontrol', '0007_create_default_logging_group'),
    ]

    operations = [
        migrations.AddField(
            model_name='loggingevent',
            name='logging_group',
            field=models.ForeignKey(default=rest_framework_apicontrol.utils.get_default_logging_group, on_delete=django.db.models.deletion.CASCADE, related_name='events', to='rest_framework_apicontrol.LoggingGroup'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-11-27 14:30
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='App',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('logo', models.URLField(blank=True, null=True)),
                ('api_key', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('enabled', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='OrganizationalUnit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('description', models.CharField(blank=True, max_length=30)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='OrganizationalUnitType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('description', models.CharField(blank=True, max_length=30)),
                ('app', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='rest_framework_apicontrol.App')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='rest_framework_apicontrol.OrganizationalUnitType')),
            ],
        ),
        migrations.CreateModel(
            name='Responsible',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=30, null=True)),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.AddField(
            model_name='organizationalunit',
            name='responsibles',
            field=models.ManyToManyField(to='rest_framework_apicontrol.Responsible'),
        ),
        migrations.AddField(
            model_name='organizationalunit',
            name='unit_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='units', to='rest_framework_apicontrol.OrganizationalUnitType'),
        ),
    ]
